@@ -22,7 +22,7 @@
 
     This file is derivative of ZynAddSubFX original code.
 
-    Modified February 2018
+    Modified July 2018
 */
 
 #include "Misc/SynthEngine.h"
@@ -375,7 +375,7 @@ float Dynamlimit::getlimits(CommandBlock *getData)
     switch (control)
     {
         case 0:
-            if (npart != 0xf1) // system effects
+            if (npart != TOPLEVEL::section::systemEffects) // system effects
                 def /= 2;
             break;
         case 1:
@@ -405,26 +405,26 @@ float Dynamlimit::getlimits(CommandBlock *getData)
             canLearn = false;
             break;
         default:
-            getData->data.type |= 4; // error
+            getData->data.type |= TOPLEVEL::type::Error;
             return 1.0f;
             break;
     }
 
-    switch(request)
+    switch (request)
     {
-        case 0:
+        case TOPLEVEL::type::Adjust:
             if(value < min)
                 value = min;
             else if(value > max)
                 value = max;
             break;
-        case 1:
+        case TOPLEVEL::type::Minimum:
             value = min;
             break;
-        case 2:
+        case TOPLEVEL::type::Maximum:
             value = max;
             break;
-        case 3:
+        case TOPLEVEL::type::Default:
             value = def;
             break;
     }
