@@ -1,5 +1,5 @@
 /*
-    EffectMgr.cpp - Effect manager, an interface betwen the program and effects
+    EffectMgr.cpp - Effect manager, an interface between the program and effects
 
     Original ZynAddSubFX author Nasca Octavian Paul
     Copyright (C) 2002-2009 Nasca Octavian Paul
@@ -162,9 +162,9 @@ void EffectMgr::changepreset_nolock(unsigned char npreset)
 // Change the preset of the current effect(with thread locking)
 void EffectMgr::changepreset(unsigned char npreset)
 {
-    synth->actionLock(lockType);
+    //synth->actionLock(lockType);
     changepreset_nolock(npreset);
-    synth->actionLock(unlockType);
+    //synth->actionLock(unlockType);
 }
 
 
@@ -180,9 +180,9 @@ void EffectMgr::seteffectpar_nolock(int npar, unsigned char value)
 // Change a parameter of the current effect (with thread locking)
 void EffectMgr::seteffectpar(int npar, unsigned char value)
 {
-    synth->actionLock(lockType);
+    //synth->actionLock(lockType);
     seteffectpar_nolock(npar, value);
-    synth->actionLock(unlockType);
+    //synth->actionLock(unlockType);
 }
 
 
@@ -347,43 +347,42 @@ void EffectMgr::getfromXML(XMLwrapper *xml)
 
 float LimitMgr::geteffectlimits(CommandBlock *getData)
 {
-    int effType = getData->data.kit & 0x7f;
-
+    int effType = getData->data.kit;
     float value = 0;
     switch (effType)
     {
-        case 0:
+        case EFFECT::type::none:
             value = 0;
             break;
-        case 1:
+        case EFFECT::type::reverb:
             Revlimit reverb;
             value = reverb.getlimits(getData);
             break;
-        case 2:
+        case EFFECT::type::echo:
             Echolimit echo;
             value = echo.getlimits(getData);
             break;
-        case 3:
+        case EFFECT::type::chorus:
             Choruslimit chorus;
             value = chorus.getlimits(getData);
             break;
-        case 4:
+        case EFFECT::type::phaser:
             Phaserlimit phaser;
             value = phaser.getlimits(getData);
             break;
-        case 5:
+        case EFFECT::type::alienWah:
             Alienlimit alien;
             value = alien.getlimits(getData);
             break;
-        case 6:
+        case EFFECT::type::distortion:
             Distlimit dist;
             value = dist.getlimits(getData);
             break;
-        case 7:
+        case EFFECT::type::eq:
             EQlimit EQ;
             value = EQ.getlimits(getData);
             break;
-        case 8:
+        case EFFECT::type::dynFilter:
             Dynamlimit dyn;
             value = dyn.getlimits(getData);
             break;
