@@ -17,7 +17,7 @@
     yoshimi; if not, write to the Free Software Foundation, Inc., 51 Franklin
     Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-    Modified March 2018
+    Modified October 2018
 */
 
 #include <iostream>
@@ -36,6 +36,8 @@ using namespace std;
 #include "Misc/XMLwrapper.h"
 #include "Misc/SynthEngine.h"
 
+enum scan : int { noList = -3, listEnd, listBlocked};
+
 MidiLearn::MidiLearn(SynthEngine *_synth) :
     learning(false),
     synth(_synth)
@@ -58,7 +60,7 @@ void MidiLearn::setTransferBlock(CommandBlock *getData, string name)
     learnedName = name;
     learning = true;
     synth->getRuntime().Log("Learning");
-    updateGui(21);
+    updateGui(MIDILEARN::control::sendLearnMessage);
 }
 
 
@@ -70,13 +72,13 @@ bool MidiLearn::runMidiLearn(int _value, unsigned int CC, unsigned char chan, un
         return true; // block while learning
     }
 
-    int lastpos = -1;
+    int lastpos = scan::listBlocked;
     LearnBlock foundEntry;
     bool firstLine = true;
-    while (lastpos != -2)
+    while (lastpos != scan::listEnd)
     {
         lastpos = findEntry(midi_list, lastpos, CC, chan, &foundEntry, false);
-        if (lastpos == -3)
+        if (lastpos == scan::noList)
             return false;
         int status = foundEntry.status;
         if (status & 4) // it's muted
@@ -150,14 +152,14 @@ bool MidiLearn::runMidiLearn(int _value, unsigned int CC, unsigned char chan, un
                 if (CC > 0xff)
                     putData.data.type |= 0x10; // mark as NRPN
                 firstLine = false;
-                putData.data.control = 24;
-                putData.data.part = 0xd8;
+                putData.data.control = MIDILEARN::control::reportActivity;
+                putData.data.part = TOPLEVEL::section::midiLearn;
                 putData.data.kit = (CC & 0xff);
                 putData.data.engine = chan;
                 writeMidi(&putData, putSize, category & 1);
             }
         }
-        if (lastpos == -1) // blocking all of this CC/chan pair
+        if (lastpos == scan::listBlocked) // blocking all of this CC/chan pair
             return true;
     }
     return false;
@@ -211,7 +213,7 @@ bool MidiLearn::writeMidi(CommandBlock *putData, unsigned int writesize, bool in
  */
 int MidiLearn::findEntry(list<LearnBlock> &midi_list, int lastpos, unsigned int CC, unsigned char chan, LearnBlock *block, bool show)
 {
-    int newpos = 0; // 'last' comes in at -1 for the first call
+    int newpos = 0; // 'last' comes in at listBlocked for the first call
     list<LearnBlock>::iterator it = midi_list.begin();
 
     while (newpos <= lastpos && it != midi_list.end())
@@ -220,7 +222,7 @@ int MidiLearn::findEntry(list<LearnBlock> &midi_list, int lastpos, unsigned int
         ++ newpos;
     }
     if (it == midi_list.end())
-        return -3;
+        return scan::noList;
 
     while ((CC != it->CC || (it->chan != 16 && chan != it->chan)) &&  it != midi_list.end())
     {
@@ -228,7 +230,7 @@ int MidiLearn::findEntry(list<LearnBlock> &midi_list, int lastpos, unsigned int
         ++ newpos;
     }
     if (it == midi_list.end())
-        return -3;
+        return scan::noList;
 
     while (CC == it->CC && it != midi_list.end())
     {
@@ -245,13 +247,13 @@ int MidiLearn::findEntry(list<LearnBlock> &midi_list, int lastpos, unsigned int
             block->max_out = it->max_out;
             block->data = it->data;
             if ((it->status & 5) == 1) // blocked, not muted
-                return -1; // don't allow any more of this CC and channel;
+                return scan::listBlocked; // don't allow any more of this CC and channel;
             return newpos;
         }
         ++ it;
         ++ newpos;
     }
-    return -2;
+    return scan::listEnd;
 }
 
 
@@ -298,7 +300,7 @@ void MidiLearn::listLine(int lineNo)
         {
             nrpn = "  NRPN";
             if (status & 16)
-                nrpn += " 7bit";
+                nrpn += " sevenBit";
         }
         string chan = "  Chan ";
         if ((it->chan) >= NUM_MIDI_CHANNELS)
@@ -369,24 +371,24 @@ void MidiLearn::generalOpps(int value, unsigned char type, unsigned char control
 {
     unsigned int kit = part; // to silence warning
     kit = _kit; // may need to set as an NRPN
-    if (control == 22)
+    if (control == MIDILEARN::control::sendRefreshRequest)
     {
         updateGui();
         synth->getRuntime().Log("GUI refreshed");
         return;
     }
 
-    if (control == 96)
+    if (control == MIDILEARN::control::clearAll)
     {
         midi_list.clear();
-        synth->setLastfileAdded(6, "");
+        synth->setLastfileAdded(XML_MIDILEARN, "");
         updateGui();
         synth->getRuntime().Log("List cleared");
         return;
     }
 
     string name;
-    if (control == 241)
+    if (control == MIDILEARN::control::loadList)
     {
         name = (miscMsgPop(par2));
         if (loadList(name))
@@ -395,10 +397,10 @@ void MidiLearn::generalOpps(int value, unsigned char type, unsigned char control
         synth->getRuntime().finishedCLI = true;
         return;
     }
-    if (control == 242) // list controls
+    if (control == MIDILEARN::control::loadFromRecent)
     {
         int pos = 0;
-        vector<string> &listtype = *synth->getHistory(6);
+        vector<string> &listtype = *synth->getHistory(XML_MIDILEARN);
         vector<string>::iterator it = listtype.begin();
         while (it != listtype.end() && pos != value)
         {
@@ -419,7 +421,7 @@ void MidiLearn::generalOpps(int value, unsigned char type, unsigned char control
         synth->getRuntime().finishedCLI = true;
         return;
     }
-    if (control == 245)
+    if (control == MIDILEARN::control::saveList)
     {
         name = (miscMsgPop(par2));
         if (saveList(name))
@@ -427,9 +429,10 @@ void MidiLearn::generalOpps(int value, unsigned char type, unsigned char control
         synth->getRuntime().finishedCLI = true;
         return;
     }
-    if (control == 255)
+    if (control == MIDILEARN::control::cancelLearn)
     {
         learning = false;
+        synth->getRuntime().finishedCLI = true;
         synth->getRuntime().Log("Midi Learn cancelled");
         updateGui(control);
         return;
@@ -480,7 +483,7 @@ void MidiLearn::generalOpps(int value, unsigned char type, unsigned char control
     }
 
 
-    if (control == 16)
+    if (control == MIDILEARN::control::CCorChannel)
     {
         bool moveLine = true;
         list<LearnBlock>::iterator nextit = it;
@@ -521,10 +524,10 @@ void MidiLearn::generalOpps(int value, unsigned char type, unsigned char control
         }
 
         if (!moveLine)
-            control = 7; // change this as we're not moving the line
+            control = MIDILEARN::control::ignoreMove; // change this as we're not moving the line
     }
 
-    if (control == 8)
+    if (control == MIDILEARN::control::deleteLine)
     {
         remove(value);
         updateGui();
@@ -532,9 +535,9 @@ void MidiLearn::generalOpps(int value, unsigned char type, unsigned char control
         return;
     }
 
-    if (control < 8)
+    if (control < MIDILEARN::control::deleteLine)
     {
-        if (control > 4)
+        if (control > MIDILEARN::control::sevenBit)
         {
             type = it->status;
             synth->getRuntime().Log("Line " + to_string(lineNo + 1) + " " + lineName);
@@ -545,19 +548,19 @@ void MidiLearn::generalOpps(int value, unsigned char type, unsigned char control
             string name;
             switch (control)
             {
-                case 0:
+                case MIDILEARN::control::block:
                     type = (tempType & 0xfe) | (type & 1);
                     name = "Block";
                     break;
-                case 1:
+                case MIDILEARN::control::limit:
                     type = (tempType & 0xfd) | (type & 2);
                     name = "Limit";
                     break;
-                case 2:
+                case MIDILEARN::control::mute:
                     type = (tempType & 0xfb) | (type & 4);
                     name = "Mute";
                     break;
-                case 4:
+                case MIDILEARN::control::sevenBit:
                     type = (tempType & 0xef) | (type & 16);
                     name = "7bit";
                     break;
@@ -573,7 +576,7 @@ void MidiLearn::generalOpps(int value, unsigned char type, unsigned char control
         // need to work on this more
         putData.data.value = value;
         putData.data.type = type;
-        putData.data.control = 7;
+        putData.data.control = MIDILEARN::control::ignoreMove;
         putData.data.kit = kit;
         putData.data.engine = engine;
         putData.data.insert = insert;
@@ -587,7 +590,7 @@ void MidiLearn::generalOpps(int value, unsigned char type, unsigned char control
         return;
     }
 
-    if (control == 16)
+    if (control == MIDILEARN::control::CCorChannel)
     {
         entry.CC = kit;
         entry.chan = engine;
@@ -645,8 +648,8 @@ void MidiLearn::insert(unsigned int CC, unsigned char chan)
         memset(&putData, 0xff, putSize);
         putData.data.value = 0;
         putData.data.type = 0xc8;
-        putData.data.control = 0xfe;
-        putData.data.part = 0xd9;
+        putData.data.control = 0xfe; // TODO don't understand this :(
+        putData.data.part = TOPLEVEL::section::midiIn;
         putData.data.parameter = 0x80;
         putData.data.par2 = miscMsgPush("Midi Learn full!");
         writeMidi(&putData, putSize, false);
@@ -713,7 +716,7 @@ void MidiLearn::insert(unsigned int CC, unsigned char chan)
     else
         CCtype = "NRPN " + asHexString((CCh >> 7) & 0x7f) + " " + asHexString(CCh & 0x7f);
     synth->getRuntime().Log(CCtype + "  Chan " + to_string((int)entry.chan + 1) + "  " + entry.name);
-    updateGui(1);
+    updateGui(MIDILEARN::control::limit);
     learning = false;
 }
 
@@ -722,7 +725,7 @@ void MidiLearn::writeToGui(CommandBlock *putData)
 {
     if (!synth->getRuntime().showGui)
         return;
-    putData->data.part = 0xd8;
+    putData->data.part = TOPLEVEL::section::midiLearn;
     unsigned int writesize = sizeof(*putData);
     char *point = (char*)putData;
     unsigned int towrite = writesize;
@@ -754,17 +757,17 @@ void MidiLearn::updateGui(int opp)
     CommandBlock putData;
     if (opp == 21)
     {
-        putData.data.control = 21;
+        putData.data.control = MIDILEARN::control::sendLearnMessage;
         putData.data.par2 = miscMsgPush("Learning " + learnedName);
     }
     else if (opp == 255)
     {
-        putData.data.control = 255;
+        putData.data.control = MIDILEARN::control::cancelLearn;
         putData.data.par2 = 0xff;
     }
     else
     {
-        putData.data.control = 96;
+        putData.data.control = MIDILEARN::control::clearAll;
         putData.data.par2 = 0xff;
         if (opp == 2)
             putData.data.kit = 2; // close editing window
@@ -783,7 +786,7 @@ void MidiLearn::updateGui(int opp)
         unsigned int newCC = it->CC;
         putData.data.value = lineNo;
         putData.data.type = it->status;
-        putData.data.control = 16;
+        putData.data.control = MIDILEARN::control::CCorChannel;
         putData.data.kit = (newCC & 0xff);
         putData.data.engine = it->chan;
         putData.data.insert = it->min_in;
@@ -792,7 +795,7 @@ void MidiLearn::updateGui(int opp)
         writeToGui(&putData);
         if (newCC > 0xff || (it->status & 8) > 0)
         { // status now used in case NRPN is < 0x100
-            putData.data.control = 9; // it's an NRPN
+            putData.data.control = MIDILEARN::control::nrpnDetected; // it's an NRPN
             putData.data.engine = ((newCC >> 8) & 0xff);
             writeToGui(&putData);
         }
@@ -801,7 +804,7 @@ void MidiLearn::updateGui(int opp)
     }
     if (opp == 1 && synth->getRuntime().showLearnedCC == true) // open the gui editing window
     {
-        putData.data.control = 22;
+        putData.data.control = MIDILEARN::control::sendRefreshRequest;
         writeToGui(&putData);
     }
 }
