@@ -4,7 +4,7 @@
     Original ZynAddSubFX author Nasca Octavian Paul
     Copyright (C) 2002-2005 Nasca Octavian Paul
     Copyright 2009-2010, Alan Calvert
-    Copyright 2014-2015, Will Godfrey & others
+    Copyright 2014-2018, Will Godfrey & others
 
     This file is part of yoshimi, which is free software: you can redistribute
     it and/or modify it under the terms of the GNU Library General Public
@@ -22,12 +22,13 @@
 
     This file is a derivative of a ZynAddSubFX original.
 
-    Modified December 2017
+    Modified September 2018
 */
 
 #include <set>
 #include <list>
 #include <stdlib.h>
+#include <stdio.h>
 #include <dirent.h>
 #include <sys/stat.h>
 #include <iostream>
@@ -51,13 +52,14 @@ using namespace std;
 Bank::Bank(SynthEngine *_synth) :
     defaultinsname(string(" ")),
     xizext(".xiz"),
+    xiyext(".xiy"),
     force_bank_dir_file(".bankdir"), // if this file exists in a directory, the
                                     // directory is considered a bank, even if
                                     // it doesn't contain an instrument file
-    synth(_synth),
-    currentRootID(0),
-    currentBankID(0)
+    synth(_synth)
 {
+    InstrumentsInBanks = 0,
+    BanksInRoots = 0;
     roots.clear();
 }
 
@@ -70,22 +72,26 @@ Bank::~Bank()
 
 string Bank::getBankFileTitle()
 {
-    return synth->makeUniqueName("Root " + asString(currentRootID) + ", Bank " + asString(currentBankID) + " - " + getBankPath(currentRootID, currentBankID));
+    return synth->makeUniqueName("Root " + asString(synth->getRuntime().currentRoot) + ", Bank " + asString(synth->getRuntime().currentBank) + " - " + getBankPath(synth->getRuntime().currentRoot, synth->getRuntime().currentBank));
 }
 
 
 string Bank::getRootFileTitle()
 {
-    return synth->makeUniqueName("Root " + asString(currentRootID) + " - " + getRootPath(currentRootID));
+    return synth->makeUniqueName("Root " + asString(synth->getRuntime().currentRoot) + " - " + getRootPath(synth->getRuntime().currentRoot));
 }
 
 
 // Get the name of an instrument from the bank
-string Bank::getname(unsigned int ninstrument)
+string Bank::getname(unsigned int ninstrument, size_t bank, size_t root)
 {
-    if (emptyslot(ninstrument))
+    if (root == 0xff)
+        root = synth->getRuntime().currentRoot;
+    if (bank == 0xff)
+        bank = synth->getRuntime().currentBank;
+    if (emptyslotWithID(root, bank, ninstrument))
         return defaultinsname;
-    return getInstrumentReference(ninstrument).name;
+    return getInstrumentReference(root, bank, ninstrument).name;
 }
 
 
@@ -95,7 +101,7 @@ string Bank::getfilename(unsigned int ninstrument)
     string fname = "";
 
     if (!emptyslot(ninstrument))
-        fname = getFullPath(currentRootID, currentBankID, ninstrument);
+        fname = getFullPath(synth->getRuntime().currentRoot, synth->getRuntime().currentBank, ninstrument);
     return fname;
 }
 
@@ -111,12 +117,21 @@ string Bank::getnamenumbered(unsigned int ninstrument)
 
 
 // Changes the name of an instrument (and the filename)
-bool Bank::setname(unsigned int ninstrument, string newname, int newslot)
+bool Bank::setname(unsigned int ninstrument, string newname, int newslot, size_t oldBank, size_t newBank, size_t oldRoot, size_t newRoot)
 {
-    if (emptyslot(ninstrument))
+    if (oldBank == 0xff)
+        oldBank = synth->getRuntime().currentBank;
+    if (oldRoot == 0xff)
+        oldRoot = synth->getRuntime().currentRoot;
+    if (newBank == 0xff)
+        newBank = oldBank;
+    if (newRoot == 0xff)
+        newRoot = oldRoot;
+
+    if (emptyslotWithID(oldRoot, oldBank, ninstrument))
         return false;
 
-    string newfilepath = getBankPath(currentRootID, currentBankID);
+    string newfilepath = getBankPath(newRoot, newBank);
     if (newfilepath.at(newfilepath.size() - 1) != '/')
         newfilepath += "/";
 
@@ -129,28 +144,19 @@ bool Bank::setname(unsigned int ninstrument, string newname, int newslot)
     int chk = -1;
     int chk2 = -1;
     newfilepath += filename;
-    string oldfilepath = setExtension(getFullPath(currentRootID, currentBankID, ninstrument), "xiz");
+    string oldfilepath = setExtension(getFullPath(oldRoot, oldBank, ninstrument), xizext);
     chk = rename(oldfilepath.c_str(), newfilepath.c_str());
-    if (chk < 0)
-    {
-        synth->getRuntime().Log("setName failed renaming "
-                + oldfilepath + " -> "
-                + newfilepath + ": " + string(strerror(errno)));
-    }
 
-    newfilepath = setExtension(newfilepath, "xiy");
-    oldfilepath = setExtension(oldfilepath, "xiy");
+    newfilepath = setExtension(newfilepath, xiyext);
+    oldfilepath = setExtension(oldfilepath, xiyext);
     chk2 = rename(oldfilepath.c_str(), newfilepath.c_str());
-    if (chk2 < 0)
-    {
-        synth->getRuntime().Log("setName failed renaming "
-                + oldfilepath + " -> "
-                + newfilepath + ": " + string(strerror(errno)));
-    }
 
     if (chk < 0 && chk2 < 0)
+    {
+        synth->getRuntime().Log("setName failed renaming " + oldfilepath + " -> " + newfilepath + ": " + string(strerror(errno)));
         return false;
-    InstrumentEntry &instrRef = getInstrumentReference(currentRootID, currentBankID, ninstrument);
+    }
+    InstrumentEntry &instrRef = getInstrumentReference(oldRoot, oldBank, ninstrument);
     instrRef.name = newname;
     instrRef.filename = filename;
     return true;
@@ -171,6 +177,12 @@ bool Bank::emptyslotWithID(size_t rootID, size_t bankID, unsigned int ninstrumen
 }
 
 
+bool Bank::emptyslot(unsigned int ninstrument)
+{
+    return emptyslotWithID(synth->getRuntime().currentRoot, synth->getRuntime().currentBank, ninstrument);
+}
+
+
 // Removes the instrument from the bank
 bool Bank::clearslot(unsigned int ninstrument)
 {
@@ -178,7 +190,7 @@ bool Bank::clearslot(unsigned int ninstrument)
     int chk2 = 0; // to stop complaints
     if (emptyslot(ninstrument))
         return true;
-    string tmpfile = setExtension(getFullPath(currentRootID, currentBankID, ninstrument), "xiy");
+    string tmpfile = setExtension(getFullPath(synth->getRuntime().currentRoot, synth->getRuntime().currentBank, ninstrument), xiyext);
 
     if (isRegFile(tmpfile))
     {
@@ -186,7 +198,7 @@ bool Bank::clearslot(unsigned int ninstrument)
         if (chk < 0)
             synth->getRuntime().Log(asString(ninstrument) + " Failed to remove " + tmpfile);
     }
-    tmpfile = setExtension(tmpfile, "xiz");
+    tmpfile = setExtension(tmpfile, xizext);
     if (isRegFile(tmpfile))
     {
         chk2 = remove(tmpfile.c_str());
@@ -196,7 +208,7 @@ bool Bank::clearslot(unsigned int ninstrument)
     if (chk < 0 || chk2 < 0)
         return false;
 
-    deletefrombank(currentRootID, currentBankID, ninstrument);
+    deletefrombank(synth->getRuntime().currentRoot, synth->getRuntime().currentBank, ninstrument);
     return true;
 }
 
@@ -230,7 +242,7 @@ bool Bank::savetoslot(size_t rootID, size_t bankID, int ninstrument, int npart)
     if (saveType & 1) // legacy
         ok2 = synth->part[npart]->saveXML(fullpath, false);
 
-    fullpath = setExtension(fullpath, "xiy");
+    fullpath = setExtension(fullpath, xiyext);
     if (isRegFile(fullpath))
     {
         int chk = remove(fullpath.c_str());
@@ -257,11 +269,13 @@ bool Bank::savetoslot(size_t rootID, size_t bankID, int ninstrument, int npart)
 
 
 //Gets a bank name
-string Bank::getBankName(int bankID)
+string Bank::getBankName(int bankID, size_t rootID)
 {
-    if (roots [currentRootID].banks.count(bankID) == 0)
+    if (rootID > 0x7f)
+        rootID = synth->getRuntime().currentRoot;
+    if (roots [rootID].banks.count(bankID) == 0)
         return "";
-    return string(roots [currentRootID].banks [bankID].dirname);
+    return string(roots [rootID].banks [bankID].dirname);
 }
 
 
@@ -276,13 +290,29 @@ string Bank::getBankIDname(int bankID)
 }
 
 
+bool Bank::isDuplicateBankName(size_t rootID, string name)
+{
+    //if(roots.count(rootID) == 0)
+        //return false;
+    for (int i = 0; i < MAX_BANK_ROOT_DIRS; ++i)
+    {
+        string check = getBankName(i,rootID);
+        if (check > "" && check == name)
+            return true;
+        if (check > "")
+            cout << check << endl;
+    }
+    return false;
+}
+
+
 // finds the number of instruments in a bank
 int Bank::getBankSize(int bankID)
 {
     int found = 0;
 
     for (int i = 0; i < BANK_SIZE; ++ i)
-        if (!roots [currentRootID].banks [bankID].instruments [i].name.empty())
+        if (!roots [synth->getRuntime().currentRoot].banks [bankID].instruments [i].name.empty())
             found += 1;
     return found;
 }
@@ -294,8 +324,8 @@ bool Bank::setbankname(unsigned int bankID, string newname)
     string filename = newname;
 
     legit_filename(filename);
-    string newfilepath = getRootPath(currentRootID) + "/" + filename;
-    int chk = rename(getBankPath(currentRootID,bankID).c_str(),
+    string newfilepath = getRootPath(synth->getRuntime().currentRoot) + "/" + filename;
+    int chk = rename(getBankPath(synth->getRuntime().currentRoot,bankID).c_str(),
                      newfilepath.c_str());
     if (chk < 0)
     {
@@ -306,7 +336,7 @@ bool Bank::setbankname(unsigned int bankID, string newname)
     synth->getRuntime().Log("Renaming " + getBankName(bankID)
                                + " to " + newname);
 
-    roots [currentRootID].banks [bankID].dirname = newname;
+    roots [synth->getRuntime().currentRoot].banks [bankID].dirname = newname;
     return true;
 }
 
@@ -345,7 +375,7 @@ bool Bank::loadbank(size_t rootID, size_t banknum)
         chkpath += candidate;
         if (isRegFile(chkpath))
         {
-            if (chkpath.rfind(".xiz") != string::npos && isRegFile(setExtension(chkpath, "xiy")))
+            if (chkpath.rfind(".xiz") != string::npos && isRegFile(setExtension(chkpath, xiyext)))
                 continue; // don't want .xiz if there is .xiy
 
             xizpos = candidate.rfind(".xiy");
@@ -383,26 +413,211 @@ bool Bank::loadbank(size_t rootID, size_t banknum)
 }
 
 
-// Makes a new bank with known ID. Does *not* make it current
-bool Bank::newIDbank(string newbankdir, unsigned int bankID)
+// Creates an external bank and copies in the contents of the IDd one
+unsigned int Bank::exportBank(string exportdir, size_t rootID, unsigned int bankID)
 {
-    if (!newbankfile(newbankdir))
+    if (rootID > 0x7f)
+        rootID = synth->getRuntime().currentRoot;
+    string name = "";
+    string sourcedir = "";
+    bool ok = true;
+    if (roots.count(rootID) == 0)
+    {
+        name = "Root ID " + to_string(int(rootID)) + " doesn't exist";
+        ok = false;
+    }
+    if (ok && roots [rootID].banks.count(bankID) == 0)
+    {
+        name = "Bank " + to_string(bankID) + " is empty";
+        ok = false;
+    }
+    else
+        sourcedir = getRootPath(rootID) + "/" + getBankName(bankID, rootID);
+
+    if (ok && isDirectory(exportdir))
+    {
+        ok = false;
+        name = "Can't overwrite existing directory";
+    }
+    if (ok)
+    {
+        int result = mkdir(exportdir.c_str(), S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);
+        if (result < 0)
+        {
+            name = "Can't create external bank " + findleafname(exportdir);
+            ok = false;
+        }
+        else
+        {
+            DIR *dir = opendir(sourcedir.c_str());
+            struct dirent *fn;
+            int count = 0;
+            int missing = 0;
+            while ((fn = readdir(dir)))
+            {
+                string nextfile = string(fn->d_name);
+                if (copyFile(sourcedir + "/" + nextfile, exportdir + "/" + nextfile))
+                    ++missing;
+                else
+                    ++count;
+            }
+            if (count > 0)
+            {
+                name = "Copied out " + to_string(count) + " files to " + exportdir + " ";
+                if (missing > 2) // seem to get 2 phantoms :(
+                    name +=( "but failed to transfer" + to_string(missing));
+            }
+            else
+            {
+                name = "to transfer to " + exportdir; // failed
+                ok = false;
+            }
+        }
+    }
+
+    unsigned int msgID = miscMsgPush(name);
+    if (!ok)
+        msgID |= 0x1000;
+    return msgID;
+}
+
+
+// Creates a new bank and copies in the contents of the external one
+unsigned int Bank::importBank(string importdir, size_t rootID, unsigned int bankID)
+{
+    if (rootID > 0x7f)
+        rootID = synth->getRuntime().currentRoot;
+    string name = "";
+    bool ok = true;
+    if (roots.count(rootID) == 0)
+    {
+        name = "Root ID " + to_string(int(rootID)) + " doesn't exist";
+        ok = false;
+    }
+
+    if (ok && roots [rootID].banks.count(bankID) != 0)
+    {
+        name = "Bank " + to_string(bankID) + " already contains " + getBankName(bankID, rootID);
+        ok = false;
+    }
+
+    if (ok)
+    {
+        DIR *dir = opendir(importdir.c_str());
+        if (dir == NULL)
+        {
+            name = "Can't find " + importdir;
+            ok = false;
+        }
+        else
+        {
+            string bankname = findleafname(importdir);
+            int repeats = 0;
+            string suffix = "";
+            while (isDirectory(getRootPath(rootID) + "/" + bankname + suffix))
+            {
+                ++repeats;
+                suffix = "~" + to_string(repeats);
+                //cout << ">" << suffix << endl;
+            }
+            bankname += suffix;
+            //cout << bankname << endl;
+            if (!newIDbank(bankname, bankID, rootID))
+            {
+                name = "Can't create bank " + bankname;
+                ok = false;
+            }
+            else
+            {
+                int count = 0;
+                int total = -2; // seem to get 2 phantoms :(
+                bool missing = false;
+                struct dirent *fn;
+                string exportfile = getRootPath(rootID) + "/" + getBankName(bankID, rootID);
+                while ((fn = readdir(dir)))
+                {
+                    string nextfile = string(fn->d_name);                    if (nextfile.rfind(".bankdir") != string::npos)
+                        continue; // new version will be generated
+                    ++total;
+                    if (nextfile.rfind(".xiy") != string::npos || nextfile.rfind(".xiz") != string::npos)
+                    {
+                        ++count;
+                        int pos = -1; // default for un-numbered
+                        int slash = nextfile.rfind("/") + 1;
+                        int hyphen = nextfile.rfind("-");
+                        if (hyphen > slash && (hyphen - slash) <= 4)
+                            pos = stoi(nextfile.substr(slash, hyphen)) - 1;
+
+                        if (copyFile(importdir + "/" + nextfile, exportfile + "/" + nextfile))
+                            missing = true;
+                        string stub;
+                        if (pos >= -1)
+                            stub = findleafname(nextfile).substr(hyphen + 1);
+                        else
+                            stub = findleafname(nextfile);
+                        if (!isDuplicate(rootID, bankID, pos, nextfile))
+                        {
+                            if (addtobank(rootID, bankID, pos, nextfile, stub))
+                                missing = true;
+                        }
+                    }
+                }
+                name = importdir;
+                if (count == 0)
+                    name += " : No valid instruments found";
+                else if (missing)
+                    name += " : Failed to copy some instruments";
+                else if (count < total)
+                    name = name + " : Ignored " + to_string(total - count)  + " unrecognised items";
+            }
+        }
+    }
+    unsigned int msgID = miscMsgPush(name);
+    if (!ok)
+        msgID |= 0x1000;
+    return msgID;
+}
+
+
+bool Bank::isDuplicate(size_t rootID, size_t bankID, int pos, const string filename)
+{
+    //cout << filename << " count " << roots [rootID].banks.count(bankID) << endl;
+    string path = getRootPath(rootID) + "/" + getBankName(bankID, rootID) + "/" + filename;
+    if (isRegFile(setExtension(path, xiyext)) && filename.rfind(xizext) < string::npos)
+        return 1;
+    if (isRegFile(setExtension(path, xizext)) && filename.rfind(xiyext) < string::npos)
+    {
+        InstrumentEntry &Ref = getInstrumentReference(rootID, bankID, pos);
+        Ref.yoshiType = true;
+        return 1;
+    }
+    return 0;
+}
+
+
+// Makes a new bank with known ID. Does *not* make it current
+bool Bank::newIDbank(string newbankdir, unsigned int bankID, size_t rootID)
+{
+    if (rootID > 0x7f)
+        rootID = synth->getRuntime().currentRoot; // should be needed!
+
+    if (!newbankfile(newbankdir, rootID))
         return false;
-    roots [currentRootID].banks [bankID].dirname = newbankdir;
-    hints [currentRootID] [newbankdir] = bankID; // why do we need this?
+    roots [synth->getRuntime().currentRoot].banks [bankID].dirname = newbankdir;
+    hints [synth->getRuntime().currentRoot] [newbankdir] = bankID; // why do we need this?
     return true;
 }
 
 
 // Performs the actual file operation for new banks
-bool Bank::newbankfile(string newbankdir)
+bool Bank::newbankfile(string newbankdir, size_t rootID)
 {
-     if (getRootPath(currentRootID).empty())
+     if (getRootPath(synth->getRuntime().currentRoot).empty())
     {
         synth->getRuntime().Log("Current bank root directory not set");
         return false;
     }
-    string newbankpath = getRootPath(currentRootID);
+    string newbankpath = getRootPath(rootID);
     if (newbankpath.at(newbankpath.size() - 1) != '/')
         newbankpath += "/";
     newbankpath += newbankdir;
@@ -426,151 +641,316 @@ bool Bank::newbankfile(string newbankdir)
 
 
 // Removes a bank and all its contents
-bool Bank::removebank(unsigned int bankID)
+unsigned int Bank::removebank(unsigned int bankID, size_t rootID)
 {
-    int chk;
+    int chk = 0;
+    if (rootID == 255)
+        rootID = synth->getRuntime().currentRoot;
+    if (roots.count(rootID) == 0)
+    {
+        chk = 0x1000 | miscMsgPush("Root " + to_string(int(rootID)) + " is empty!");
+        return chk;
+    }
+    string bankName = getBankPath(rootID, bankID);
+    string IDfile = bankName + "/.bankdir";
+    FILE *tmpfile = fopen(IDfile.c_str(), "w+");
+    if (!tmpfile)
+        chk = 0x1000 | miscMsgPush("Can't delete from this location.");
+    else
+        fclose(tmpfile);
 
+    int ck1 = 0;
+    int ck2 = 0;
+    string name;
     for (int inst = 0; inst < BANK_SIZE; ++ inst)
     {
-        if (!roots [currentRootID].banks [bankID].instruments [inst].name.empty())
+        if (!roots [rootID].banks [bankID].instruments [inst].name.empty())
         {
-            chk = remove(getFullPath(currentRootID, bankID, inst).c_str());
-            if (chk < 0)
-            {
-                synth->getRuntime().Log(asString(inst) + " Failed to remove "
-                                        + getFullPath(currentRootID, bankID, inst) + " "
-                                        + string(strerror(errno)), 2);
-                return false;
-            }
-            deletefrombank(currentRootID, bankID, inst);
+            name = setExtension(getFullPath(synth->getRuntime().currentRoot, bankID, inst), xiyext);
+            if (isRegFile(name))
+                ck1 = remove(name.c_str());
+            else
+                ck1 = 0;
+
+            name = setExtension(name, xizext);
+            if (isRegFile(name))
+                ck2 = remove(name.c_str());
+            else
+                ck2 = 0;
+
+            if (ck1 == 0 && ck2 == 0)
+                deletefrombank(rootID, bankID, inst);
+            else if (chk == 0) // only want to name one entry
+                    chk = 0x1000 | miscMsgPush(findleafname(name) + ". Others may also still exist.");
         }
     }
-    string tmp = getBankPath(currentRootID, bankID)+"/.bankdir";
-    if (!access(tmp.c_str(), W_OK))
-    {
-        chk = remove(tmp.c_str());
-        if (chk < 0)
+    if (chk > 0)
+        return chk;
+
+    if (isRegFile(IDfile))
+    { // only removed when bank cleared
+        if (remove(IDfile.c_str()) != 0)
         {
-            synth->getRuntime().Log("Failed to remove bank ID file "
-                                    + string(strerror(errno)), 2);
-            return false;
+            chk = 0x1000 | miscMsgPush(findleafname(name));
+            return chk;
         }
     }
-    chk = remove(getBankPath(currentRootID, bankID).c_str());
-    if (chk < 0)
+
+    if (remove(bankName.c_str()) != 0)
     {
-        synth->getRuntime().Log("Failed to remove bank"
-                                + asString(bankID) + ": "
-                                + string(strerror(errno)), 2);
-        return false;
+        chk = 0x1000 | miscMsgPush(bankName + ". Unrecognised contents still exist.");
+        return chk;
     }
-    roots [currentRootID].banks.erase(bankID);
-    return true;
+
+    roots [rootID].banks.erase(bankID);
+    if (rootID == synth->getRuntime().currentRoot && bankID == synth->getRuntime().currentBank)
+        setCurrentBankID(0);
+    chk = miscMsgPush(bankName);
+    return chk;
 }
 
 
 // Swaps a slot with another
-bool Bank::swapslot(unsigned int n1, unsigned int n2)
+unsigned int Bank::swapslot(unsigned int n1, unsigned int n2, size_t bank1, size_t bank2, size_t root1, size_t root2)
 {
-    if (n1 == n2)
+    if (n1 == n2 && bank1 == bank2 && root1 == root2)
         return true;
+    if (bank1 == 255)
+        bank1 = synth->getRuntime().currentBank;
+    if (bank2 == 255)
+        bank2 = bank1;
+    if (root1 == 255)
+        root1 = synth->getRuntime().currentRoot;
+    if (root2 == 255)
+        root2 = root1;
 
-    if (emptyslot(n1) && emptyslot(n2))
-        return true;
-    if (emptyslot(n1)) // make the empty slot the destination
-    {
-        if (!setname(n2, getname(n2), n1))
-            return false;
-        getInstrumentReference(n1) = getInstrumentReference(n2);
-        getInstrumentReference(n2).clear();
-    }
-    else if (emptyslot(n2)) // this is just a movement to an empty slot
-    {
-        if (!setname(n1, getname(n1), n2))
-            return false;
-        getInstrumentReference(n2) = getInstrumentReference(n1);
-        getInstrumentReference(n1).clear();
-    }
-    else
-    {   // if both slots are used
-        InstrumentEntry &instrRef1 = getInstrumentReference(n1);
-        InstrumentEntry &instrRef2 = getInstrumentReference(n2);
-        if (instrRef1.name == instrRef2.name)
+    cout << "first " << getname(n1, bank1, root1) << "   second " << getname(n2, bank2, root2) << endl;
+    /*
+     * path entries will always have either .xiy or .xiz
+     * otherwise they would not have been seen at all
+     * however we test for, and move both if they exist
+     */
+    cout << "first ref" << getFullPath(root1, bank1, n1) << endl;
+    string message = "";
+
+    if (emptyslotWithID(root1, bank1, n1) && emptyslotWithID(root2, bank2, n2))
+        message = "nothing to swap!";
+
+    else if (emptyslotWithID(root1, bank1, n1) || emptyslotWithID(root2, bank2, n2))
+    { // this is just a movement to an empty slot
+        if (emptyslotWithID(root1, bank1, n1)) // make the empty slot the destination
         {
-            // change the name of the second instrument if the name are equal
-            instrRef2.name += "2";
+            if (!setname(n2, getname(n2, bank2, root2), n1, bank2, bank1, root2, root1))
+            {
+                message = "can't write to " + getname(n2, bank2, root2);
+            }
+            getInstrumentReference(root1, bank1, n1) = getInstrumentReference(root2, bank2, n2);
+            getInstrumentReference(root2, bank2, n2).clear();
         }
-        if (!setname(n2, getname(n2), n1))
-            return false;
-        if (!setname(n1, getname(n1), n2))
-            return false;
-        InstrumentEntry instrTmp = instrRef1;
-        instrRef1 = instrRef2;
-        instrRef2 = instrTmp;
+        else
+        {
+            if (!setname(n1, getname(n1, bank1, root1), n2, bank1, bank2, root1, root2))
+            {
+                message = "can't write to " + getname(n1, bank1, root1);
+            }
+            getInstrumentReference(root2, bank2, n2) = getInstrumentReference(root1, bank1, n1);
+            getInstrumentReference(root1, bank1, n1).clear();
+        }
+        if (message > "")
+            return miscMsgPush(message) | 0x1000;
+        else
+            return 0;
     }
-    return true;
+
+    // if both slots are used
+    string firstName = getname(n1, bank1, root1);
+    string secondName = getname(n2, bank2, root2);
+    if (firstName == secondName)
+        message = "can't swap instruments with identical names.";
+    else
+    {
+        InstrumentEntry &instrRef1 = getInstrumentReference(root1, bank1, n1);
+        InstrumentEntry &instrRef2 = getInstrumentReference(root2, bank2, n2);
+
+        if (!setname(n2, secondName, n1, bank2, bank1, root2, root1))
+            message = "can't change " + secondName;
+
+        if (!setname(n1, firstName, n2, bank1, bank2, root1, root2))
+            message = "can't change " + firstName;
+        {
+            InstrumentEntry instrTmp = instrRef1;
+            instrRef1 = instrRef2;
+            instrRef2 = instrTmp;
+        }
+    }
+    if (message > "")
+        return miscMsgPush(message) | 0x1000;
+    return 0;
 }
 
 
 // Intelligently moves or swaps banks preserving instrument details
-void Bank::swapbanks(unsigned int firstID, unsigned int secondID)
+unsigned int Bank::swapbanks(unsigned int firstID, unsigned int secondID, size_t firstRoot, size_t secondRoot)
 {
-    if (firstID == secondID)
-    {
-        synth->getRuntime().Log("Nothing to move!");
-        return;
-    }
+    unsigned int result = 0;
+    string message = "";
+    string firstname;
+    string secondname;
 
-    string firstname = getBankName(firstID); // this needs improving
-    string secondname = getBankName(secondID);
-    if (firstname.empty() and secondname.empty())
+    if (firstRoot > 0x7f)
+        firstRoot = synth->getRuntime().currentRoot;
+    if (secondRoot > 0x7f)
+        secondRoot = firstRoot;
+
+    if (firstID == secondID && firstRoot == secondRoot)
     {
-        synth->getRuntime().Log("Nothing to move!");
-        return;
-    }
-    if (secondname.empty())
-    {
-        synth->getRuntime().Log("Moving " + firstname);
-        roots [currentRootID].banks [secondID] = roots [currentRootID].banks [firstID];
-        roots [currentRootID].banks.erase(firstID);
-    }
-    else if (firstname.empty())
-    {
-        synth->getRuntime().Log("Moving " + secondname);
-        roots [currentRootID].banks [firstID] = roots [currentRootID].banks [secondID];
-        roots [currentRootID].banks.erase(secondID);
+        message = "nothing to swap!";
+        result = 0x1000;
     }
     else
     {
-        synth->getRuntime().Log("Swapping " + firstname + " with " + secondname );
-        roots [currentRootID].banks [firstID].dirname = secondname;
-        roots [currentRootID].banks [secondID].dirname = firstname;
-        hints [currentRootID] [secondname] = firstID; // why do we need these?
-        hints [currentRootID] [firstname] = secondID;
-
-        for(int pos = 0; pos < BANK_SIZE; ++ pos)
+        firstname = getBankName(firstID, firstRoot);
+        secondname = getBankName(secondID, secondRoot);
+        if (firstname.empty() && secondname.empty())
         {
-            InstrumentEntry &instrRef_1 = getInstrumentReference(currentRootID, firstID, pos);
-            InstrumentEntry &instrRef_2 = getInstrumentReference(currentRootID, secondID, pos);
-
-            InstrumentEntry tmp = instrRef_2;
-
-            if (instrRef_1.name == "")
-                roots [currentRootID].banks [secondID].instruments.erase(pos);
-            else
-                instrRef_2 = instrRef_1;
-
-            if (tmp.name == "")
-                roots [currentRootID].banks [firstID].instruments.erase(pos);
-            else
-                instrRef_1 = tmp;
+            message = "nothing to swap!";
+            result = 0x1000;
         }
     }
 
-    if (firstID == currentBankID)
-        currentBankID = secondID;
-    else if(secondID == currentBankID)
-        currentBankID = firstID;
+    if (firstRoot != secondRoot)
+    {
+        if (isDuplicateBankName(firstRoot, secondname))
+        {
+            message = secondname + " already exists in root " + to_string(firstRoot);
+            result = 0x1000;
+        }
+        else if (isDuplicateBankName(secondRoot, firstname))
+        {
+            message = firstname + " already exists in root " + to_string(secondRoot);
+            result = 0x1000;
+        }
+    }
+
+    if (result == 0 && firstRoot != secondRoot) // do physical move first
+    {
+        string firstBankPath = getBankPath(firstRoot, firstID);
+        string secondBankPath = getBankPath(secondRoot, secondID);
+        string newfirstBankPath = getRootPath(secondRoot) + "/" + firstname;
+        string newsecondBankPath = getRootPath(firstRoot) + "/" + secondname;
+        string tempBankPath = getRootPath(firstRoot) + "/tempfile";
+        if (secondBankPath == "") // move only
+        {
+            result = rename(firstBankPath.c_str(), (getRootPath(secondRoot) + "/" + firstname).c_str());
+            if (result != 0)
+            {
+                synth->getRuntime().Log("move to " + to_string(secondRoot) + ": " + string(strerror(errno)), 2);
+                message = "Can't move from root " + to_string(firstRoot) + " to " + to_string(secondRoot);
+                result = 0x1000;
+            }
+        }
+        else if(firstBankPath == "") // move only
+        {
+            result = rename(secondBankPath.c_str(), (getRootPath(firstRoot) + "/" + secondname).c_str());
+            if (result != 0)
+            {
+                synth->getRuntime().Log("move to " + to_string(firstRoot) + ": " + string(strerror(errno)), 2);
+                message = "Can't move from root " + to_string(secondRoot) + " to " + to_string(firstRoot);
+                result = 0x1000;
+            }
+        }
+        else // actual swap
+        {
+            // due to possible identical names we need to go via a temp file
+
+            //cout << "first " << firstBankPath << endl;
+            //cout << "second " << secondBankPath << endl;
+            //cout << "newfirst " << newfirstBankPath << endl;
+            //cout << "newsecond " << newsecondBankPath << endl;
+            result = remove(tempBankPath.c_str()); // just to be sure
+            result = rename(firstBankPath.c_str(), tempBankPath.c_str());
+            if (result == 0)
+            {
+                result = rename(secondBankPath.c_str(),newsecondBankPath.c_str());
+                if (result == 0)
+                {
+                    result = rename(tempBankPath.c_str(), newfirstBankPath.c_str());
+                    if (result != 0)
+                    {
+                        synth->getRuntime().Log("move to " + to_string(secondRoot) + ": " + string(strerror(errno)), 2);
+                        message = "Can't move from temp dir to " + to_string(secondRoot);
+                        result = 0x1000;
+                    }
+                }
+                else
+                {
+                    synth->getRuntime().Log("move to " + to_string(firstRoot) + ": " + string(strerror(errno)), 2);
+                    message = "Can't move from root " + to_string(secondRoot) + " to " + to_string(firstRoot);
+                    result = 0x1000;
+                }
+            }
+            else
+            {
+                synth->getRuntime().Log("move to temp dir: " + string(strerror(errno)), 2);
+                message = "Can't move from root " + to_string(firstRoot) + " to temp dir";
+                result = 0x1000;
+            }
+        }
+    }
+
+    if (result == 0) // update banks
+    {
+        if (secondname.empty())
+        {
+            roots [secondRoot].banks [secondID] = roots [firstRoot].banks [firstID];
+            roots [firstRoot].banks.erase(firstID);
+        }
+        else if (firstname.empty())
+        {
+            roots [firstRoot].banks [firstID] = roots [secondRoot].banks [secondID];
+            roots [secondRoot].banks.erase(secondID);
+        }
+        else
+        {
+            roots [firstRoot].banks [firstID].dirname = secondname;
+            roots [secondRoot].banks [secondID].dirname = firstname;
+            hints [secondRoot] [secondname] = firstID; // why do we need these?
+            hints [firstRoot] [firstname] = secondID;
+
+            for(int pos = 0; pos < BANK_SIZE; ++ pos)
+            {
+                InstrumentEntry &instrRef_1 = getInstrumentReference(firstRoot, firstID, pos);
+                InstrumentEntry &instrRef_2 = getInstrumentReference(secondRoot, secondID, pos);
+
+                InstrumentEntry tmp = instrRef_2;
+
+                if (instrRef_1.name == "")
+                    roots [secondRoot].banks [secondID].instruments.erase(pos);
+                else
+                    instrRef_2 = instrRef_1;
+
+                if (tmp.name == "")
+                    roots [firstRoot].banks [firstID].instruments.erase(pos);
+                else
+                    instrRef_1 = tmp;
+            }
+        }
+    }
+
+    if (result == 0)
+    {
+        if (firstRoot == synth->getRuntime().currentRoot)
+            synth->getRuntime().currentRoot = secondRoot;
+        else if(secondRoot == synth->getRuntime().currentBank)
+            synth->getRuntime().currentBank = firstRoot;
+        if (firstID == synth->getRuntime().currentBank)
+            synth->getRuntime().currentBank = secondID;
+        else if(secondID == synth->getRuntime().currentBank)
+            synth->getRuntime().currentBank = firstID;
+    }
+
+    result |= miscMsgPush(message);
+    return result;
 }
 
 
@@ -578,11 +958,13 @@ void Bank::swapbanks(unsigned int firstID, unsigned int secondID)
 void Bank::rescanforbanks(void)
 {
     RootEntryMap::const_iterator it;
-
+    InstrumentsInBanks = 0;
+    BanksInRoots = 0;
     for (it = roots.begin(); it != roots.end(); ++it)
     {
         scanrootdir(it->first);
     }
+    //cout << "ins " << InstrumentsInBanks << "  Ban " << BanksInRoots << endl;
 }
 
 // private affairs
@@ -662,6 +1044,7 @@ void Bank::scanrootdir(int root_idx)
     for(it = bankDirsMap.begin(); it != bankDirsMap.end(); ++it)
     {
         add_bank(it->first, it->second, root_idx);
+        BanksInRoots += 1;
     }
     roots [root_idx].bankIdStep = 0;
 }
@@ -717,10 +1100,10 @@ bool Bank::addtobank(size_t rootID, size_t bankID, int pos, const string filenam
     // see which engines are used
     if (synth->getRuntime().checksynthengines)
     {
-        string checkfile = setExtension(getFullPath(rootID, bankID, pos), "xiy");
+        string checkfile = setExtension(getFullPath(rootID, bankID, pos), xiyext);
         if (!isRegFile(checkfile))
-            checkfile = setExtension(getFullPath(rootID, bankID, pos), "xiz");
-        XMLwrapper *xml = new XMLwrapper(synth);
+            checkfile = setExtension(getFullPath(rootID, bankID, pos), xizext);
+        XMLwrapper *xml = new XMLwrapper(synth, true, false);
         xml->checkfileinformation(checkfile);
         instrRef.PADsynth_used = xml->information.PADsynth_used;
         instrRef.ADDsynth_used = xml->information.ADDsynth_used;
@@ -728,6 +1111,7 @@ bool Bank::addtobank(size_t rootID, size_t bankID, int pos, const string filenam
         instrRef.yoshiType = xml->information.yoshiType;
         delete xml;
     }
+    InstrumentsInBanks += 1;
     return 0;
 }
 
@@ -771,7 +1155,7 @@ size_t Bank::add_bank(string name, string , size_t rootID)
 
 InstrumentEntry &Bank::getInstrumentReference(size_t ninstrument)
 {
-    return getInstrumentReference(currentRootID, currentBankID, ninstrument);
+    return getInstrumentReference(synth->getRuntime().currentRoot, synth->getRuntime().currentBank, ninstrument);
 }
 
 InstrumentEntry &Bank::getInstrumentReference(size_t rootID, size_t bankID, size_t ninstrument)
@@ -788,6 +1172,7 @@ void Bank::addDefaultRootDirs()
         "/usr/share/zynaddsubfx/banks",
         "/usr/local/share/zynaddsubfx/banks",
         string(getenv("HOME")) + "/banks",
+        localPath("/banks"),
         "end"
     };
     int i = 0;
@@ -797,7 +1182,6 @@ void Bank::addDefaultRootDirs()
         addRootDir(bankdirs [i]);
         ++ i;
     }
-    addRootDir(localPath("/banks"));
 
     while ( i >= 0)
     {
@@ -930,7 +1314,7 @@ const RootEntryMap &Bank::getRoots()
 
 const BankEntry &Bank::getBank(size_t bankID)
 {
-    return roots [currentRootID].banks [bankID];
+    return roots [synth->getRuntime().currentRoot].banks [bankID];
 }
 
 
@@ -952,12 +1336,12 @@ void Bank::clearBankrootDirlist(void)
 
 void Bank::removeRoot(size_t rootID)
 {
-    if(rootID == currentRootID)
+    if(rootID == synth->getRuntime().currentRoot)
     {
-        currentRootID = 0;
+        synth->getRuntime().currentRoot = 0;
     }
     roots.erase(rootID);
-    setCurrentRootID(currentRootID);
+    setCurrentRootID(synth->getRuntime().currentRoot);
 }
 
 
@@ -994,14 +1378,13 @@ bool Bank::setCurrentRootID(size_t newRootID)
         }
         else
         {
-            currentRootID = roots.begin()->first;
+            synth->getRuntime().currentRoot = roots.begin()->first;
         }
     }
     else
     {
-        currentRootID = newRootID;
+        synth->getRuntime().currentRoot = newRootID;
     }
-
     setCurrentBankID(0);
     return true;
 }
@@ -1009,22 +1392,29 @@ bool Bank::setCurrentRootID(size_t newRootID)
 
 bool Bank::setCurrentBankID(size_t newBankID, bool ignoreMissing)
 {
-    if(roots [currentRootID].banks.count(newBankID) == 0)
+    if(roots [synth->getRuntime().currentRoot].banks.count(newBankID) == 0)
     {
-        if((roots [currentRootID].banks.size() == 0) || ignoreMissing)
+        if((roots [synth->getRuntime().currentRoot].banks.size() == 0) || ignoreMissing)
         {
             return false;
         }
         else
         {
-            newBankID = roots [currentRootID].banks.begin()->first;
+            newBankID = roots [synth->getRuntime().currentRoot].banks.begin()->first;
         }
     }
-    currentBankID = newBankID;
+    synth->getRuntime().currentBank = newBankID;
     return true;
 }
 
 
+size_t Bank::getCurrentBankID()
+{// This is only used by the root section of BankUI
+    return synth->getRuntime().currentBank;
+
+}
+
+
 size_t Bank::addRootDir(string newRootDir)
 {
    // we need the size check to prevent weird behaviour if the name is just ./
