@@ -5,7 +5,7 @@
     Copyright (C) 2002-2005 Nasca Octavian Paul
     Copyright 2009-2011, Alan Calvert
     Copyright 2013, Nikita Zlobin
-    Copyright 2014-2017, Will Godfrey & others
+    Copyright 2014-2018, Will Godfrey & others
 
     This file is part of yoshimi, which is free software: you can redistribute
     it and/or modify it under the terms of the GNU Library General Public
@@ -23,7 +23,7 @@
 
     This file is derivative of ZynAddSubFX original code.
 
-    Modified September 2017
+    Modified November 2018
 */
 
 #include <iostream>
@@ -49,7 +49,7 @@ using namespace std;
 #include "Misc/SynthEngine.h"
 #include "Misc/Config.h"
 #include "MasterUI.h"
-#include "ConfBuild.cpp"
+#include "ConfBuild.h"
 
 static char prog_doc[] =
     "Yoshimi " YOSHIMI_VERSION ", a derivative of ZynAddSubFX - "
@@ -57,33 +57,33 @@ static char prog_doc[] =
     "Copyright 2009-2011 Alan Calvert, "
     "Copyright 20012-2013 Jeremy Jongepier and others, "
     "Copyright 20014-2017 Will Godfrey and others";
-string argline = "Yoshimi " + (string) YOSHIMI_VERSION + "\nBuild Number " + to_string(BUILD_NUMBER);
+string argline = "Yoshimi " + (string) YOSHIMI_VERSION;// + "\nBuild Number " + to_string(BUILD_NUMBER);
 const char* argp_program_version = argline.c_str();
 
 static struct argp_option cmd_options[] = {
-    {"alsa-audio",        'A',  "<device>",   1,  "use alsa audio output" },
-    {"alsa-midi",         'a',  "<device>",   1,  "use alsa midi input" },
-    {"define-root",       'D',  "<path>",     0,  "define path to new bank root"},
-    {"buffersize",        'b',  "<size>",     0,  "set internal buffer size" },
-    {"no-gui",            'i',  NULL,         0,  "disable gui"},
-    {"gui",               'I',  NULL,         0,  "enable gui"},
-    {"no-cmdline",        'c',  NULL,         0,  "disable command line interface"},
-    {"cmdline",           'C',  NULL,         0,  "enable command line interface"},
-    {"jack-audio",        'J',  "<server>",   1,  "use jack audio output" },
-    {"jack-midi",         'j',  "<device>",   1,  "use jack midi input" },
-    {"autostart-jack",    'k',  NULL,         0,  "auto start jack server" },
-    {"auto-connect",      'K',  NULL,         0,  "auto connect jack audio" },
-    {"load",              'l',  "<file>",     0,  "load .xmz file" },
-    {"load-instrument",   'L',  "<file>",     0,  "load .xiz file" },
-    {"name-tag",          'N',  "<tag>",      0,  "add tag to clientname" },
-    {"samplerate",        'R',  "<rate>",     0,  "set alsa audio sample rate" },
-    {"oscilsize",         'o',  "<size>",     0,  "set AddSynth oscilator size" },
-    {"state",             'S',  "<file>",     1,  "load saved state, defaults to '$HOME/.config/yoshimi/yoshimi.state'" },
+    {"alsa-audio",        'A',  "<device>",   1,  "use alsa audio output", 0},
+    {"alsa-midi",         'a',  "<device>",   1,  "use alsa midi input", 0},
+    {"define-root",       'D',  "<path>",     0,  "define path to new bank root" , 0},
+    {"buffersize",        'b',  "<size>",     0,  "set internal buffer size", 0 },
+    {"no-gui",            'i',  NULL,         0,  "disable gui", 0},
+    {"gui",               'I',  NULL,         0,  "enable gui", 0},
+    {"no-cmdline",        'c',  NULL,         0,  "disable command line interface", 0},
+    {"cmdline",           'C',  NULL,         0,  "enable command line interface", 0},
+    {"jack-audio",        'J',  "<server>",   1,  "use jack audio output", 0},
+    {"jack-midi",         'j',  "<device>",   1,  "use jack midi input", 0},
+    {"autostart-jack",    'k',  NULL,         0,  "auto start jack server", 0},
+    {"auto-connect",      'K',  NULL,         0,  "auto connect jack audio", 0},
+    {"load",              'l',  "<file>",     0,  "load .xmz file", 0},
+    {"load-instrument",   'L',  "<file>",     0,  "load .xiz file", 0},
+    {"name-tag",          'N',  "<tag>",      0,  "add tag to clientname", 0},
+    {"samplerate",        'R',  "<rate>",     0,  "set alsa audio sample rate", 0},
+    {"oscilsize",         'o',  "<size>",     0,  "set AddSynth oscilator size", 0},
+    {"state",             'S',  "<file>",     1,  "load saved state, defaults to '$HOME/.config/yoshimi/yoshimi.state'", 0},
     #if defined(JACK_SESSION)
-        {"jack-session-uuid", 'U',  "<uuid>",     0,  "jack session uuid" },
-        {"jack-session-file", 'u',  "<file>",     0,  "load named jack session file" },
+        {"jack-session-uuid", 'U',  "<uuid>",     0,  "jack session uuid", 0},
+        {"jack-session-file", 'u',  "<file>",     0,  "load named jack session file", 0},
     #endif
-    { 0, }
+    { 0, 0, 0, 0, 0, 0}
 };
 
 unsigned int Config::Samplerate = 48000;
@@ -93,11 +93,15 @@ unsigned int Config::GzipCompression = 3;
 bool         Config::showGui = true;
 bool         Config::showSplash = true;
 bool         Config::showCLI = true;
+bool         Config::autoInstance = false;
+unsigned int Config::activeInstance = 0;
+int          Config::showCLIcontext = 1;
 
 Config::Config(SynthEngine *_synth, int argc, char **argv) :
     restoreState(false),
     stateChanged(false),
     restoreJackSession(false),
+    oldConfig(false),
     runSynth(true),
     finishedCLI(true),
     VirKeybLayout(0),
@@ -135,6 +139,11 @@ Config::Config(SynthEngine *_synth, int argc, char **argv) :
     single_row_panel(1),
     NumAvailableParts(NUM_MIDI_CHANNELS),
     currentPart(0),
+    currentBank(0),
+    currentRoot(0),
+    tempBank(0),
+    tempRoot(0),
+    VUcount(0),
     channelSwitchType(0),
     channelSwitchCC(128),
     channelSwitchValue(0),
@@ -160,6 +169,12 @@ Config::Config(SynthEngine *_synth, int argc, char **argv) :
     //as all calls to lrintf() are replaced with (int)truncf()
     //which befaves exactly the same when flag FE_TOWARDZERO is set
 
+    /*
+     * The above is now all completely redundant as we use
+     * in-line either fast assembly (where available) or the
+     * original Zyn float - int conversion rounding to zero
+     */
+
     cerr.precision(4);
     bRuntimeSetupCompleted = Setup(argc, argv);
 }
@@ -178,10 +193,10 @@ bool Config::Setup(int argc, char **argv)
         /*
          * These are needed here now, as for stand-alone they have
          * been moved to main to give the users the impression of
-         * a faster startup, and reduce the likelyhood of thinking
+         * a faster startup, and reduce the likelihood of thinking
          * they failed and trying to start again.
          */
-        synth->installBanks(synth->getUniqueId());
+        synth->installBanks();
         synth->loadHistory();
         return true;
     }
@@ -220,32 +235,29 @@ bool Config::Setup(int argc, char **argv)
     if (!midiDevice.size())
         midiDevice = "";
     loadCmdArgs(argc, argv);
-    Oscilsize = nearestPowerOf2(Oscilsize, MAX_AD_HARMONICS * 2, 16384);
-    Buffersize = nearestPowerOf2(Buffersize, 16, 4096);
+    Oscilsize = nearestPowerOf2(Oscilsize, MIN_OSCIL_SIZE, MAX_OSCIL_SIZE);
+    Buffersize = nearestPowerOf2(Buffersize, MIN_BUFFER_SIZE, MAX_BUFFER_SIZE);
     //Log(asString(Oscilsize));
     //Log(asString(Buffersize));
-    if (loadDefaultState && !restoreState)
-    {
-        StateFile = ConfigDir + "/yoshimi.state";
-        restoreState = true;
-    }
+
     if (restoreState)
     {
-        char * fp;
-        if (! StateFile.size()) goto no_state0;
-        else fp = new char [PATH_MAX];
+        char *fp = NULL;
+        if (!StateFile.size())
+            goto no_state;
+
+        fp = realpath (StateFile.c_str(), NULL);
+        if (fp == NULL)
+            goto no_state;
 
-        if (! realpath (StateFile.c_str(), fp)) goto no_state1;
         StateFile = fp;
-        delete (fp);
-
-        if (! isRegFile(StateFile))
+        free (fp);
+        if (!isRegFile(StateFile))
         {
-            no_state1: delete (fp);
-            no_state0: Log("Invalid state file specified for restore " + StateFile, 2);
+            no_state: Log("Invalid state file specified for restore " + StateFile, 2);
             return true;
         }
-        Log(StateFile);
+        Log("Using " + StateFile);
         restoreSessionData(StateFile, true);
         /* There is a single state file that contains both startup config
          * data that must be set early, and runtime data that must be set
@@ -425,11 +437,12 @@ bool Config::loadConfig(void)
             return false;
         }
     }
-    string yoshimi = "/"; // for some reason it doesn't
-    yoshimi += YOSHIMI; // like these as one line here
+    string yoshimi = "/" + string(YOSHIMI);
 
-    if (synth->getUniqueId() > 0)
-        yoshimi += ("-" + asString(synth->getUniqueId()));
+    string baseConfig = ConfigDir + yoshimi + ".config";
+    int thisInstance = synth->getUniqueId();
+    if (thisInstance > 0)
+        yoshimi += ("-" + asString(thisInstance));
     else
         miscMsgInit(); // sneaked it in here so it's early
     string presetDir = ConfigDir + "/presets";
@@ -439,43 +452,61 @@ bool Config::loadConfig(void)
         if ((chk = system(cmd.c_str())) < 0)
             Log("Create preset directory " + presetDir + " failed, status " + asString(chk));
     }
+
     ConfigFile = ConfigDir + yoshimi;
     StateFile = ConfigDir + yoshimi + string(".state");
-    if (synth->getUniqueId() == 0)
-        ConfigFile += ".config";
+
+    if (thisInstance == 0)
+        ConfigFile = baseConfig;
     else
         ConfigFile += ".instance";
-    string resConfigFile = ConfigFile;
 
+    if (!isRegFile(baseConfig))
+    {
+        Log("Basic configuration " + baseConfig + " not found, will use default settings");
+        defaultPresets();
+    }
 
     bool isok = true;
-    if (!isRegFile(resConfigFile) && !isRegFile(ConfigFile))
+    if (!isRegFile(ConfigFile))
     {
-        Log("ConfigFile " + resConfigFile + " not found, will use default settings");
-        defaultPresets();
+        Log("Configuration " + ConfigFile + " not found, will use default settings");
         configChanged = true; // give the user the choice
     }
     else
     {
-        XMLwrapper *xml = new XMLwrapper(synth);
+        XMLwrapper *xml = new XMLwrapper(synth, true);
         if (!xml)
             Log("loadConfig failed XMLwrapper allocation");
         else
         {
-            if (!xml->loadXMLfile(resConfigFile))
+            if (!xml->loadXMLfile(baseConfig))
             {
-                if ((synth->getUniqueId() > 0) && (!xml->loadXMLfile(ConfigFile)))
+                if (thisInstance > 0)
                 {
                     Log("loadConfig loadXMLfile failed");
                     return false;
                 }
             }
             isok = extractBaseParameters(xml);
-            if (isok)
-                isok = extractConfigData(xml);
-            if (isok)
-                Oscilsize = (int)truncf(powf(2.0f, ceil(log (Oscilsize - 1.0f) / logf(2.0))));
             delete xml;
+            if (isok)
+            {
+                XMLwrapper *xml = new XMLwrapper(synth, true);
+                isok = xml->loadXMLfile(ConfigFile);
+                if (isok)
+                {
+                    isok = extractConfigData(xml);
+                    delete xml;
+                }
+            }
+            if (thisInstance == 0)
+            {
+                if (lastXMLmajor < MIN_CONFIG_MAJOR || lastXMLminor < MIN_CONFIG_MINOR)
+                    oldConfig = true;
+                else
+                    oldConfig = false;
+            }
         }
     }
     return isok;
@@ -522,12 +553,15 @@ bool Config::extractBaseParameters(XMLwrapper *xml)
         return false;
     }
     Samplerate = xml->getpar("sample_rate", Samplerate, 44100, 192000);
-    Buffersize = xml->getpar("sound_buffer_size", Buffersize, 16, 4096);
-    Oscilsize = xml->getpar("oscil_size", Oscilsize, MAX_AD_HARMONICS * 2, 16384);
+    Buffersize = xml->getpar("sound_buffer_size", Buffersize, MIN_BUFFER_SIZE, MAX_BUFFER_SIZE);
+    Oscilsize = xml->getpar("oscil_size", Oscilsize, MIN_OSCIL_SIZE, MAX_OSCIL_SIZE);
     GzipCompression = xml->getpar("gzip_compression", GzipCompression, 0, 9);
     showGui = xml->getparbool("enable_gui", showGui);
     showSplash = xml->getparbool("enable_splash", showSplash);
     showCLI = xml->getparbool("enable_CLI", showCLI);
+    autoInstance = xml->getparbool("enable_auto_instance", autoInstance);
+    activeInstance = xml->getparU("active_instances", 0);
+    showCLIcontext = xml->getpar("show_CLI_context", 1, 0, 2);
     xml->exitbranch(); // BaseParameters
     return true;
 }
@@ -605,9 +639,12 @@ bool Config::extractConfigData(XMLwrapper *xml)
 
     //misc
     checksynthengines = xml->getpar("check_pad_synth", checksynthengines, 0, 1);
-    tempRoot = xml->getpar("root_current_ID", 0, 0, 127);
+    if (tempRoot == 0)
+        tempRoot = xml->getpar("root_current_ID", 0, 0, 127);
+    //else
+        //cout << "root? " << xml->getpar("root_current_ID", 0, 0, 127) << endl;
+    if (tempBank == 0)
     tempBank = xml->getpar("bank_current_ID", 0, 0, 127);
-
     xml->exitbranch(); // CONFIGURATION
     return true;
 }
@@ -617,7 +654,7 @@ bool Config::saveConfig(void)
 {
     bool result = false;
     xmlType = XML_CONFIG;
-    XMLwrapper *xmltree = new XMLwrapper(synth);
+    XMLwrapper *xmltree = new XMLwrapper(synth, true);
     if (!xmltree)
     {
         Log("saveConfig failed xmltree allocation", 2);
@@ -693,7 +730,7 @@ bool Config::saveSessionData(string savefile)
 {
     savefile = setExtension(savefile, "state");
     synth->getRuntime().xmlType = XML_STATE;
-    XMLwrapper *xmltree = new XMLwrapper(synth);
+    XMLwrapper *xmltree = new XMLwrapper(synth, true);
     if (!xmltree)
     {
         Log("saveSessionData failed xmltree allocation", 3);
@@ -728,7 +765,7 @@ bool Config::restoreSessionData(string sessionfile, bool startup)
         Log("Session file " + sessionfile + " not available", 2);
         goto end_game;
     }
-    if (!(xml = new XMLwrapper(synth)))
+    if (!(xml = new XMLwrapper(synth, true)))
     {
         Log("Failed to init xmltree for restoreState", 3);
         goto end_game;
@@ -786,11 +823,16 @@ void Config::LogError(const string &msg)
     Log("[ERROR] " + msg, 1);
 }
 
-#ifndef YOSHIMI_LV2_PLUGIN
-void Config::StartupReport(MusicClient *musicClient)
+//#ifndef YOSHIMI_LV2_PLUGIN
+void Config::StartupReport(string clientName)
 {
-    Log(string(argp_program_version));
-    Log("Clientname: " + musicClient->midiClientName());
+    bool fullInfo = (synth->getUniqueId() == 0);
+    if (fullInfo)
+    {
+        Log(argline);
+        Log("Build Number " + to_string(BUILD_NUMBER), 1);
+    }
+    Log("Clientname: " + clientName);
     string report = "Audio: ";
     switch (audioEngine)
     {
@@ -825,11 +867,14 @@ void Config::StartupReport(MusicClient *musicClient)
         midiDevice = "default";
     report += (" -> '" + midiDevice + "'");
     Log(report, 2);
-    Log("Oscilsize: " + asString(synth->oscilsize), 2);
-    Log("Samplerate: " + asString(synth->samplerate), 2);
-    Log("Period size: " + asString(synth->buffersize), 2);
+    if (fullInfo)
+    {
+        Log("Oscilsize: " + asString(synth->oscilsize), 2);
+        Log("Samplerate: " + asString(synth->samplerate), 2);
+        Log("Period size: " + asString(synth->buffersize), 2);
+    }
 }
-#endif
+//#endif
 
 
 void Config::setRtprio(int prio)
@@ -879,7 +924,7 @@ bool Config::startThread(pthread_t *pth, void *(*thread_fn)(void*), void *arg,
                     int prio = rtprio - priodec;
                     if (prio < 1)
                         prio = 1;
-                    Log(name + " priority is " + to_string(prio), 2);
+                    Log(name + " priority is " + to_string(prio), 1);
                     prio_params.sched_priority = prio;
                     if ((chk = pthread_attr_setschedparam(&attr, &prio_params)))
                     {
@@ -1211,12 +1256,11 @@ static error_t parse_cmds (int key, char *arg, struct argp_state *state)
         default:
             return error_t(ARGP_ERR_UNKNOWN);
     }
-
     return error_t(0);
 }
 
 
-static struct argp cmd_argp = { cmd_options, parse_cmds, prog_doc };
+static struct argp cmd_argp = { cmd_options, parse_cmds, prog_doc, 0, 0, 0, 0};
 
 
 void Config::loadCmdArgs(int argc, char **argv)
@@ -1260,51 +1304,11 @@ void GuiThreadMsg::processGuiMessages()
                     guiMaster->updatepaths(msg->index);
                     break;
 
-                case GuiThreadMsg::UpdatePanel:
-                    guiMaster->updatepanel();
-                    break;
-
                 case GuiThreadMsg::UpdatePart:
                     guiMaster->updatepart();
                     guiMaster->updatepanel();
                     break;
 
-                case GuiThreadMsg::UpdatePanelItem:
-                    if ( msg->data && msg->index < NUM_MIDI_PARTS)
-                    {
-                        guiMaster->updatelistitem(msg->index);
-                        guiMaster->updatepart();
-                    }
-                    break;
-
-                case GuiThreadMsg::UpdatePartProgram:
-                    if (msg->data && msg->index < NUM_MIDI_PARTS)
-                    {
-                        guiMaster->updatelistitem(msg->index);
-                        guiMaster->updatepartprogram(msg->index);
-                    }
-                    break;
-
-                case GuiThreadMsg::UpdateEffects:
-                    if (msg->data)
-                        guiMaster->updateeffects(msg->index);
-                    break;
-
-                case GuiThreadMsg::UpdateControllers:
-                    if (msg->data)
-                        guiMaster->updatecontrollers(msg->index);
-                    break;
-
-                case GuiThreadMsg::UpdateBankRootDirs:
-                    if (msg->data)
-                        guiMaster->updateBankRootDirs();
-                    break;
-
-                case GuiThreadMsg::RescanForBanks:
-                    if (msg->data && guiMaster->bankui)
-                        guiMaster->bankui->rescan_for_banks(false);
-                    break;
-
                 case GuiThreadMsg::RefreshCurBank:
                     if (msg->data && guiMaster->bankui)
                     {
@@ -1319,9 +1323,8 @@ void GuiThreadMsg::processGuiMessages()
                     }
                     break;
 
-                case GuiThreadMsg::GuiAlert:
-                    if (msg->data)
-                        guiMaster->ShowAlert(msg->index);
+                case GuiThreadMsg::GuiCheck:
+                    guiMaster->checkBuffer();
                     break;
 
                 default:
