@@ -4,7 +4,7 @@
     Original ZynAddSubFX author Nasca Octavian Paul
     Copyright (C) 2002-2005 Nasca Octavian Paul
     Copyright 2009-2011 Alan Calvert
-    Copyright 2014-2017, Will Godfrey
+    Copyright 2014-2018, Will Godfrey
 
     This file is part of yoshimi, which is free software: you can redistribute
     it and/or modify it under the terms of the GNU Library General Public
@@ -22,7 +22,7 @@
 
     This file is derivative of original ZynAddSubFX code.
 
-    Modified December 2017
+    Modified February 2018
 */
 
 #ifndef PART_H
@@ -52,7 +52,7 @@ class SynthEngine;
 class Part : private MiscFuncs, SynthHelper
 {
     public:
-        enum NoteStatus { KEY_OFF, KEY_PLAYING, KEY_RELASED_AND_SUSTAINED, KEY_RELASED };
+        enum NoteStatus { KEY_OFF, KEY_PLAYING, KEY_RELEASED_AND_SUSTAINED, KEY_RELEASED };
 
         Part(Microtonal *microtonal_, FFTwrapper *fft_, SynthEngine *_synth);
         ~Part();
@@ -61,7 +61,6 @@ class Part : private MiscFuncs, SynthHelper
         void defaults(void);
         void setNoteMap(int keyshift);
         void defaultsinstrument(void);
-        void applyparameters(void);
         void cleanup(void);
 
         // Midi commands implemented
@@ -69,8 +68,8 @@ class Part : private MiscFuncs, SynthHelper
         void NoteOff(int note);
         void AllNotesOff(void) { killallnotes = true; }; // panic, prepare all notes to be turned off
         void SetController(unsigned int type, int par);
-        void RelaseSustainedKeys(void);
-        void RelaseAllKeys(void);
+        void ReleaseSustainedKeys(void);
+        void ReleaseAllKeys(void);
         void ComputePartSmps(void);
 
         bool saveXML(string filename, bool yoshiFormat); // result true for load ok, otherwise false
@@ -79,7 +78,7 @@ class Part : private MiscFuncs, SynthHelper
         void add2XMLinstrument(XMLwrapper *xml);
         void getfromXML(XMLwrapper *xml);
         void getfromXMLinstrument(XMLwrapper *xml);
-        void getLimits(CommandBlock *getData);
+        float getLimits(CommandBlock *getData);
 
         Controller *ctl;
 
@@ -156,11 +155,12 @@ class Part : private MiscFuncs, SynthHelper
         float pangainL;
         float pangainR;
         int lastnote;
+        bool busy;
 
 
     private:
         void KillNotePos(int pos);
-        void RelaseNotePos(int pos);
+        void ReleaseNotePos(int pos);
         void MonoMemRenote(void); // MonoMem stuff.
         void setPan(float value);
 
