@@ -22,7 +22,7 @@
     Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
     This file is derivative of ZynAddSubFX original code
-    Modified March 2018
+    Modified October 2018
 */
 
 #include <cmath>
@@ -55,7 +55,8 @@ SUBnote::SUBnote(SUBnoteParameters *parameters, Controller *ctl_, float freq,
 
     // Initialise some legato-specific vars
     Legato.msg = LM_Norm;
-    Legato.fade.length = (int)truncf(synth->samplerate_f * 0.005f); // 0.005 seems ok.
+    FR2Z2I(synth->samplerate_f * 0.005f, Legato.fade.length); // 0.005 seems ok.
+    //Legato.fade.length = (int)truncf(synth->samplerate_f * 0.005f); // 0.005 seems ok.
     if (Legato.fade.length < 1)
         Legato.fade.length = 1;// (if something's fishy)
     Legato.fade.step = (1.0f / Legato.fade.length);
@@ -797,16 +798,16 @@ int SUBnote::noteout(float *outl, float *outr)
 }
 
 
-// Relase Key (Note Off)
-void SUBnote::relasekey(void)
+// Release Key (Note Off)
+void SUBnote::releasekey(void)
 {
-    AmpEnvelope->relasekey();
+    AmpEnvelope->releasekey();
     if (FreqEnvelope != NULL)
-        FreqEnvelope->relasekey();
+        FreqEnvelope->releasekey();
     if (BandWidthEnvelope != NULL)
-        BandWidthEnvelope->relasekey();
+        BandWidthEnvelope->releasekey();
     if (GlobalFilterEnvelope != NULL)
-        GlobalFilterEnvelope->relasekey();
+        GlobalFilterEnvelope->releasekey();
 }
 
 void SUBnote::initfilterbank(void)
